#!/usr/bin/perl
$Usage = "$0 [-m n_max_download/-nbr][-n new_body][-referer][-a(create_body_from_url)] body_with_[start-end/start+cnt]_*\n";

#--- Default settings of timeout
$DnsTimeout = 5;
$ConnectTimeout = 5;
$ReadTimeout = 300;
$max_number_download = 10;
$need_referer = 1;		# to add the basic referer.

#--- interpret the command-line arguments
while (defined($ARGV[0])) {
    if ($ARGV[0] =~ /^-n.*/) {
	shift(@ARGV);
	$new_body = $ARGV[0];
	$new_body =~ s|/|_|g;
    } elsif ($ARGV[0] =~ /^-a.*/) {
	$create_new_body_from_url = 1;
    } elsif ($ARGV[0] =~ /^-m.*/) {
	shift(@ARGV);
	$max_number_download = $ARGV[0];;
    } elsif ($ARGV[0] =~ /^-s.*/) {
	$delay_multiplier = 2;
    } elsif ($ARGV[0] =~ /^-r.*/) {
	$need_referer = 0;
    } elsif ($ARGV[0] =~ /^-D.*/) {
        $Debug = 1;
    } elsif ($ARGV[0] =~ /^-[0-9]*/) {
	$n_download = $ARGV[0];
	$n_download =~ s/^-//;
    } elsif ($ARGV[0] =~ /^-.*/) {
	print STDERR "$Usage";
	exit 0;
    } else {
	$body = $ARGV[0];
    }
    shift(@ARGV);
}

$Options = "-b --dns-timeout=$DnsTimeout --connect-timeout=$ConnectTimeout --read-timeout=$ReadTimeout -o /dev/null";

#--- DebugPrint function
# $Debug = 1;
sub Dprint { if (defined($Debug)) { printf STDERR "[DBG] @_"; } return 0; }

Dprint "$body / $new_body\n";

#--- retrieve start, end and body from the body argument.
$logfile_name = CreateLogfileName();
if (!defined($body)) {
    print STDERR "$Usage";
    exit 0;
}

#-- file is assumed to have the structure "body12[34-56]tail" or "body1234tail".
if ($body =~ /\[.*\]/) {
    # url and start/end is specified by the "head[kako]tail" notation.
    # kako = (start-end) or (start + plus_number)
    $head = $tail = $kako = $body;
    $_ = $body;
    m|^(.*)\[([0-9\-\+]*)\](.*)$|;
    $head  = $1;
    $kako  = $2;
    $tail  = $3;
    if ($tail =~ /\/$/) {
	$dummy_tail = $tail;
	$dummy_tail =~ s|/|_|g;
	$dummy_tail =~ s|[-_]$||;
	$dummy_tail .= ".any";
    } else {
	undef($dummy_tail);
    }
    Dprint "tail[$tail] dummy_tail[$dummy_tail] head[$head] start[$start]\n";
    if ($kako =~ /\+/) {
	$kako =~ m|([0-9]+)\+([0-9]+)|;
	$start = $1;
	$end   = $start + $2 - 1;
    } else {
	$kako =~ m|([0-9]+)-([0-9]+)|;
	$start = $1;
	$end   = $2;
    }
} else {
    # url has assuming structure "{head}{number}{tail}".
    # nbr of download is specified by the other argument.
    $head = $tail = $start = $body;
    $_ = $body;
    m|^(.*[^0-9])([0-9]+)([^0-9]+)$|;
    $head  = $1;
    $start = $2;
    $tail  = $3;
    if ($tail =~ /\/$/) {
	$dummy_tail = $tail;
	$dummy_tail =~ s|/|_|g;
	$dummy_tail .= ".any";
    } else {
	undef($dummy_tail);
    }
    Dprint "tail[$tail] dummy_tail[$dummy_tail] head[$head] start[$start]\n";
    if (!defined($n_download)) {
	$ext = $tail;
	$ext =~ s/^.*\.//;
	Dprint "EXT = $ext\n";
	if ($ext eq "jpg" || $ext eq "JPG") {
	    $n_download = 18;
	} elsif ($ext eq "wmv" || $ext eq "WMV" ||
		 $ext eq "avi" || $ext eq "AVI" ||
		 $ext eq "mpg" || $ext eq "MPG" ||
		 $ext eq "flv" || $ext eq "FLV") {
	    $n_download = 4;
	} else {
	    $n_download = 1;
	}
    }
    $end = $start + $n_download - 1;
    while (length($end) < length($start)) {
	$end = "0" . $end;
    }
}
Dprint "BODY: body=($body) head=($head) start=($start) end=($end) / tail=($tail)\n";    
if ($start =~ /^0[0-9][0-9]*$/) {
    $width = (length($start) > length($end)) ? length($start) : length($end);
}

#--- checking to set the necessary variables.
if (!(defined($head) || defined($tail)) || !defined($start) || !defined($end)) {
    print STDERR "$Usage";
    exit 0;
}

if (defined($need_referer) && $need_referer) {
    $rfr = CreateReferer($body);
    $Options = "--referer=$rfr $Options";
}

#--- change body to create strings.
open(OUT, ">>$logfile_name") or print STDERR "Can't open $logfile_name\n";
for ($n = $start; $n <= $end; $n++) {
    sleep_if_busy();

    if (defined($width)) {
	$ns = sprintf(($width == 0) ? "%d" : "%0${width}d", $n);
	if (defined($create_new_body_from_url)) {
	    $new_body = create_new_body($body, $width);
	    Dprint "body($body) -> new($new_body)\n";
	}
    } else {
	$ns = $n;
	if (defined($create_new_body_from_url)) {
	    $new_body = create_new_body($body, 1);
	    Dprint "body($body) -> new($new_body)\n";
	}
    }

    #--- download URL
    $b = $head . $ns . $tail;

    #--- name if the downloaded file.
    if (defined($new_body)) {
	if (defined($dummy_tail)) {
	    $newn = $new_body . $ns . $dummy_tail;
	} else {
	    $newn = $new_body . $ns . $tail;
	}
	Dprint "[b][$b] $new[$newn]\n";
	$newn =~ s|$head|$new_body|;
	Dprint "nice -15 wget $Options -O $newn \"$b\"\n";
	print "[$b]->[$newn]\n";
	`nice -15 wget $Options -O $newn \"$b\"`;
	if ($? == 0) {
	    print OUT "$newn\t$b\n";
	}
    } else {
	Dprint "nice -15 wget $Options \"$b\"\n";
	print "[$b]-> LOCAL\n";
	`nice -15 wget $Options \"$b\"`;
	if ($? == 0) {
	    print OUT "$1\t$b\n";
	}
    }

    if (!defined($cnt)) {
	$cnt = 1;
    } else {
	$cnt++;
    }
}
close(OUT);
exit 0;

sub sleep_awhile
{
    local($nbr);
    $nbr = $_[0];
    if ($nbr % 40 == 0) {
	sleep(defined($delay_multiplier) ? 8 * $delay_multiplier: 8);
    } elsif ($nbr % 20 == 0) {
	sleep(defined($delay_multiplier) ? 4 * $delay_multiplier: 4);
    } elsif ($nbr % 10 == 0) {
	sleep(defined($delay_multiplier) ? 1 * $delay_multiplier: 2);
    }
    return;
}

sub sleep_if_busy
{
    local($n);
    local($sleep_time) = 1;
    $nbr = $_[0];
    while (($n = number_wget()) >= $max_number_download) {
	Dprint "sleep_if_busy: nwget = $n, sleep_time = $sleep_time\n";
	sleep($sleep_time);
	$sleep_time = $sleep_time >= 8 ? 10 : $sleep_time * 2;
    }
    return;
}

sub number_wget
{
    local($n) = 0;
    local(@p);
    local(@c);
    local($cmd);
    open(IN, "ps x |");
    while(<IN>) {
	@p = split();
	@c = split(/\//, $p[4]);
	$cmd = pop(@c);
	# Dprint "$p[4] : $cmd\n";
	if ($cmd eq wget) {
	    $n++;
	}
    }
    close(IN);
    return($n);
}

sub create_new_body
{
    my $nb = $_[0], $nlt, $wd = $_[1], $i;
    $nb =~ s|//|/|g;
    $nb =~ s|\[[0-9\-\+]*\]||;
    $nb =~ s|[_\-][_\-]*|_|g;
    if ($nb =~ m|(.*)\.[^./]*$|) {
	$nb = $1;
    }
    for ($i = 0; $i < $wd; $i++) {
	$nb =~ s|[0-9]$||;
    }
    $nb =~ s|\.|_|g;
    $nb =~ s|[_\-]$||;
    @nlt = split(/\//, $nb);
    # Dprint "create_new_body[$_[0]][$nb][@nlt]\n";
    if ($#nlt > 2) {
	$nb = $nlt[$#nlt - 2] . "_" . $nlt[$#nlt - 1] . "_" . $nlt[$#nlt];
    } elsif ($#nlt > 1) {
	$nb = $nlt[$#nlt - 1] . "_" . $nlt[$#nlt];
    } else {
	$nb = $nlt[$#nlt];
    }
    return ($nb);
} 

sub CreateLogfileName
{
    my $lgn;
    $_ = $0;
    m|([^/]*)$|;
    $_ = $1;
    s|\.[^.]*$||;
    s|\.|_|g;
    $lgn = ".${_}-$$" . time . ".url";
    Dprint "CreateLogfileName [$lgn]\n";
    return $lgn;
}

sub CreateReferer
{
    my $rfr = $_[0];
    $rfr =~ s|(http://[^/]*).*|$1|;
    $rfr = "${rfr}/";
    Dprint "Referer: $rfr";
    return $rfr;
}
